@@ -4,7 +4,7 @@ use 5.008;
 use strict;
 use warnings;
 
-our $VERSION = '0.08_1';
+our $VERSION = '0.03';
 
 =head1 NAME
 
@@ -17,14 +17,9 @@ Search::Estraier - pure perl module to use Hyper Estraier search engine
 	use Search::Estraier;
 
 	# create and configure node
-	my $node = new Search::Estraier::Node(
-		url => 'http://localhost:1978/node/test',
-		user => 'admin',
-		passwd => 'admin',
-		create => 1,
-		label => 'Label for node',
-		croak_on_error => 1,
-	);
+	my $node = new Search::Estraier::Node;
+	$node->set_url("http://localhost:1978/node/test");
+	$node->set_auth("admin","admin");
 
 	# create document
 	my $doc = new Search::Estraier::Document;
@@ -37,19 +32,16 @@ Search::Estraier - pure perl module to use Hyper Estraier search engine
 	$doc->add_text("Somewhere over the rainbow.  Way up high.");
 	$doc->add_text("There's a land that I heard of once in a lullaby.");
 
-	die "error: ", $node->status,"\n" unless (eval { $node->put_doc($doc) });
+	die "error: ", $node->status,"\n" unless ($node->put_doc($doc));
 
 =head2 Simple searcher
 
 	use Search::Estraier;
 
 	# create and configure node
-	my $node = new Search::Estraier::Node(
-		url => 'http://localhost:1978/node/test',
-		user => 'admin',
-		passwd => 'admin',
-		croak_on_error => 1,
-	);
+	my $node = new Search::Estraier::Node;
+	$node->set_url("http://localhost:1978/node/test");
+	$node->set_auth("admin","admin");
 
 	# create condition
 	my $cond = new Search::Estraier::Condition;
@@ -58,10 +50,7 @@ Search::Estraier - pure perl module to use Hyper Estraier search engine
 	$cond->set_phrase("rainbow AND lullaby");
 
 	my $nres = $node->search($cond, 0);
-
 	if (defined($nres)) {
-		print "Got ", $nres->hits, " results\n";
-
 		# for each document in results
 		for my $i ( 0 ... $nres->doc_num - 1 ) {
 			# get result document
@@ -103,8 +92,7 @@ Remove multiple whitespaces from string, as well as whitespaces at beginning or
 =cut
 
 sub _s {
-	my $text = $_[1];
-	return unless defined($text);
+	my $text = $_[1] || return;
 	$text =~ s/\s\s+/ /gs;
 	$text =~ s/^\s+//;
 	$text =~ s/\s+$//;
@@ -120,34 +108,9 @@ our @ISA = qw/Search::Estraier/;
 
 =head1 Search::Estraier::Document
 
-This class implements Document which is single item in Hyper Estraier.
+This class implements Document which is collection of attributes
+(key=value), vectors (also key value) display text and hidden text.
 
-It's is collection of:
-
-=over 4
-
-=item attributes
-
-C<< 'key' => 'value' >> pairs which can later be used for filtering of results
-
-You can add common filters to C<attrindex> in estmaster's C<_conf>
-file for better performance. See C<attrindex> in
-L<Hyper Estraier P2P Guide|http://hyperestraier.sourceforge.net/nguide-en.html>.
-
-=item vectors
-
-also C<< 'key' => 'value' >> pairs
-
-=item display text
-
-Text which will be used to create searchable corpus of your index and
-included in snippet output.
-
-=item hidden text
-
-Text which will be searchable, but will not be included in snippet.
-
-=back
 
 =head2 new
 
@@ -182,15 +145,11 @@ sub new {
 
 			if ($line =~ m/^%VECTOR\t(.+)$/) {
 				my @fields = split(/\t/, $1);
-				if ($#fields % 2 == 1) {
-					$self->{kwords} = { @fields };
-				} else {
-					warn "can't decode $line\n";
+				for my $i ( 0 .. ($#fields - 1) ) {
+					$self->{kwords}->{ $fields[ $i ] } = $fields[ $i + 1 ];
+					$i++;
 				}
 				next;
-			} elsif ($line =~ m/^%SCORE\t(.+)$/) {
-			    $self->{score} = $1;
-			    next;
 			} elsif ($line =~ m/^%/) {
 				# What is this? comment?
 				#warn "$line\n";
@@ -198,12 +157,12 @@ sub new {
 			} elsif ($line =~ m/^$/) {
 				$in_text = 1;
 				next;
-			} elsif ($line =~ m/^(.+)=(.*)$/) {
+			} elsif ($line =~ m/^(.+)=(.+)$/) {
 				$self->{attrs}->{ $1 } = $2;
 				next;
 			}
 
-			warn "draft ignored: '$line'\n";
+			warn "draft ignored: $line\n";
 		}
 	}
 
@@ -272,53 +231,6 @@ sub add_hidden_text {
 	push @{ $self->{htexts} }, $self->_s($text);
 }
 
-=head2 add_vectors
-
-Add a vectors
-
-  $doc->add_vector(
-  	'vector_name' => 42,
-	'another' => 12345,
-  );
-
-=cut
-
-sub add_vectors {
-	my $self = shift;
-	return unless (@_);
-
-	# this is ugly, but works
-	die "add_vector needs HASH as argument" unless ($#_ % 2 == 1);
-
-	$self->{kwords} = {@_};
-}
-
-=head2 set_score
-
-Set the substitute score
-
-  $doc->set_score(12345);
-
-=cut
-
-sub set_score {
-    my $self = shift;
-    my $score = shift;
-    return unless (defined($score));
-    $self->{score} = $score;
-}
-
-=head2 score
-
-Get the substitute score
-
-=cut
-
-sub score {
-    my $self = shift;
-    return -1 unless (defined($self->{score}));
-    return $self->{score};
-}
 
 =head2 id
 
@@ -408,29 +320,22 @@ sub dump_draft {
 	my $draft;
 
 	foreach my $attr_name (sort keys %{ $self->{attrs} }) {
-		next unless defined(my $v = $self->{attrs}->{$attr_name});
-		$draft .= $attr_name . '=' . $v . "\n";
+		$draft .= $attr_name . '=' . $self->{attrs}->{$attr_name} . "\n";
 	}
 
 	if ($self->{kwords}) {
-		$draft .= '%VECTOR';
+		$draft .= '%%VECTOR';
 		while (my ($key, $value) = each %{ $self->{kwords} }) {
 			$draft .= "\t$key\t$value";
 		}
 		$draft .= "\n";
 	}
 
-	if (defined($self->{score}) && $self->{score} >= 0) {
-	    $draft .= "%SCORE\t" . $self->{score} . "\n";
-	}
-
 	$draft .= "\n";
 
 	$draft .= join("\n", @{ $self->{dtexts} }) . "\n" if ($self->{dtexts});
 	$draft .= "\t" . join("\n\t", @{ $self->{htexts} }) . "\n" if ($self->{htexts});
 
-	printf("[%s]\n", $draft);
-
 	return $draft;
 }
 
@@ -463,7 +368,7 @@ sub delete {
 
 package Search::Estraier::Condition;
 
-use Carp qw/carp confess croak/;
+use Carp qw/confess croak/;
 
 use Search::Estraier;
 our @ISA = qw/Search::Estraier/;
@@ -541,71 +446,30 @@ sub set_max {
 
 =head2 set_options
 
-  $cond->set_options( 'SURE' );
-
-  $cond->set_options( qw/AGITO NOIDF SIMPLE/ );
-
-Possible options are:
-
-=over 8
-
-=item SURE
-
-check every N-gram
-
-=item USUAL
-
-check every second N-gram
-
-=item FAST
-
-check every third N-gram
-
-=item AGITO
-
-check every fourth N-gram
-
-=item NOIDF
-
-don't perform TF-IDF tuning
-
-=item SIMPLE
-
-use simplified query phrase
-
-=back
-
-Skipping N-grams will speed up search, but reduce accuracy. Every call to C<set_options> will reset previous
-options;
-
-This option changed in version C<0.04> of this module. It's backwards compatibile.
+  $cond->set_options( SURE => 1 );
 
 =cut
 
 my $options = {
+	# check N-gram keys skipping by three
 	SURE => 1 << 0,
+	# check N-gram keys skipping by two
 	USUAL => 1 << 1,
+	# without TF-IDF tuning
 	FAST => 1 << 2,
+	# with the simplified phrase
 	AGITO => 1 << 3,
+	# check every N-gram key
 	NOIDF => 1 << 4,
+	# check N-gram keys skipping by one
 	SIMPLE => 1 << 10,
 };
 
 sub set_options {
 	my $self = shift;
-	my $opt = 0;
-	foreach my $option (@_) {
-		my $mask;
-		unless ($mask = $options->{$option}) {
-			if ($option eq '1') {
-				next;
-			} else {
-				croak "unknown option $option";
-			}
-		}
-		$opt += $mask;
-	}
-	$self->{options} = $opt;
+	my $option = shift;
+	confess "unknown option" unless ($options->{$option});
+	$self->{options} ||= $options->{$option};
 }
 
 
@@ -684,76 +548,6 @@ sub options {
 }
 
 
-=head2 set_skip
-
-Set number of skipped documents from beginning of results
-
-  $cond->set_skip(42);
-
-Similar to C<offset> in RDBMS.
-
-=cut
-
-sub set_skip {
-	my $self = shift;
-	$self->{skip} = shift;
-}
-
-=head2 skip
-
-Return skip for this condition.
-
-  print $cond->skip;
-
-=cut
-
-sub skip {
-	my $self = shift;
-	return $self->{skip};
-}
-
-
-=head2 set_distinct
-
-  $cond->set_distinct('@author');
-
-=cut
-
-sub set_distinct {
-	my $self = shift;
-	$self->{distinct} = shift;
-}
-
-=head2 distinct
-
-Return distinct attribute
-
-  print $cond->distinct;
-
-=cut
-
-sub distinct {
-	my $self = shift;
-	return $self->{distinct};
-}
-
-=head2 set_mask
-
-Filter out some links when searching.
-
-Argument array of link numbers, starting with 0 (current node).
-
-  $cond->set_mask(qw/0 1 4/);
-
-=cut
-
-sub set_mask {
-	my $self = shift;
-	return unless (@_);
-	$self->{mask} = \@_;
-}
-
-
 package Search::Estraier::ResultDocument;
 
 use Carp qw/croak/;
@@ -897,9 +691,6 @@ Return number of documents
 
   print $res->doc_num;
 
-This will return real number of documents (limited by C<max>).
-If you want to get total number of hits, see C<hits>.
-
 =cut
 
 sub doc_num {
@@ -931,7 +722,7 @@ sub get_doc {
 
 Return specific hint from results.
 
-  print $res->hint( 'VERSION' );
+  print $rec->hint( 'VERSION' );
 
 Possible hints are: C<VERSION>, C<NODE>, C<HIT>, C<HINT#n>, C<DOCNUM>, C<WORDNUM>,
 C<TIME>, C<LINK#n>, C<VIEW>.
@@ -944,37 +735,6 @@ sub hint {
 	return $self->{hints}->{$key};
 }
 
-=head2 hints
-
-More perlish version of C<hint>. This one returns hash.
-
-  my %hints = $res->hints;
-
-=cut
-
-sub hints {
-	my $self = shift;
-	return $self->{hints};
-}
-
-=head2 hits
-
-Syntaxtic sugar for total number of hits for this query
-
-  print $res->hits;
-
-It's same as
-
-  print $res->hint('HIT');
-
-but shorter.
-
-=cut
-
-sub hits {
-	my $self = shift;
-	return $self->{hints}->{'HIT'} || 0;
-}
 
 package Search::Estraier::Node;
 
@@ -994,54 +754,6 @@ or optionally with C<url> as parametar
 
   my $node = new Search::HyperEstraier::Node( 'http://localhost:1978/node/test' );
 
-or in more verbose form
-
-  my $node = new Search::HyperEstraier::Node(
-  	url => 'http://localhost:1978/node/test',
-	user => 'admin',
-	passwd => 'admin'
-	create => 1,
-	label => 'optional node label',
-	debug => 1,
-	croak_on_error => 1
-  );
-
-with following arguments:
-
-=over 4
-
-=item url
-
-URL to node
-
-=item user
-
-specify username for node server authentication
-
-=item passwd
-
-password for authentication
-
-=item create
-
-create node if it doesn't exists
-
-=item label
-
-optional label for new node if C<create> is used
-
-=item debug
-
-dumps a B<lot> of debugging output
-
-=item croak_on_error
-
-very helpful during development. It will croak on all errors instead of
-silently returning C<-1> (which is convention of Hyper Estraier API in other
-languages).
-
-=back
-
 =cut
 
 sub new {
@@ -1049,43 +761,25 @@ sub new {
 	my $self = {
 		pxport => -1,
 		timeout => 0,	# this used to be -1
+		dnum => -1,
+		wnum => -1,
+		size => -1.0,
 		wwidth => 480,
 		hwidth => 96,
 		awidth => 96,
 		status => -1,
 	};
-
 	bless($self, $class);
 
 	if ($#_ == 0) {
 		$self->{url} = shift;
 	} else {
-		%$self = ( %$self, @_ );
-
-		$self->set_auth( $self->{user}, $self->{passwd} ) if ($self->{user});
+		my $args = {@_};
 
+		$self->{debug} = $args->{debug};
 		warn "## Node debug on\n" if ($self->{debug});
 	}
 
-	$self->{inform} = {
-		dnum => -1,
-		wnum => -1,
-		size => -1.0,
-	};
-
-	if ($self->{create}) {
-		if (! eval { $self->name } || $@) {
-			my $name = $1 if ($self->{url} =~ m#/node/([^/]+)/*#);
-			croak "can't find node name in '$self->{url}'" unless ($name);
-			my $label = $self->{label} || $name;
-			$self->master(
-				action => 'nodeadd',
-				name => $name,
-				label => $label,
-			) || croak "can't create node $name ($label)";
-		}
-	}
-
 	$self ? return $self : return undef;
 }
 
@@ -1176,7 +870,7 @@ Add a document
 
   $node->put_doc( $document_draft ) or die "can't add document";
 
-Return true on success or false on failure.
+Return true on success or false on failture.
 
 =cut
 
@@ -1184,15 +878,11 @@ sub put_doc {
 	my $self = shift;
 	my $doc = shift || return;
 	return unless ($self->{url} && $doc->isa('Search::Estraier::Document'));
-	if ($self->shuttle_url( $self->{url} . '/put_doc',
+	$self->shuttle_url( $self->{url} . '/put_doc',
 		'text/x-estraier-draft',
 		$doc->dump_draft,
 		undef
-	) == 200) {
-		$self->_clear_info;
-		return 1;
-	}
-	return undef;
+	) == 200;
 }
 
 
@@ -1211,15 +901,11 @@ sub out_doc {
 	my $id = shift || return;
 	return unless ($self->{url});
 	croak "id must be number, not '$id'" unless ($id =~ m/^\d+$/);
-	if ($self->shuttle_url( $self->{url} . '/out_doc',
+	$self->shuttle_url( $self->{url} . '/out_doc',
 		'application/x-www-form-urlencoded',
 		"id=$id",
 		undef
-	) == 200) {
-		$self->_clear_info;
-		return 1;
-	}
-	return undef;
+	) == 200;
 }
 
 
@@ -1237,15 +923,11 @@ sub out_doc_by_uri {
 	my $self = shift;
 	my $uri = shift || return;
 	return unless ($self->{url});
-	if ($self->shuttle_url( $self->{url} . '/out_doc',
+	$self->shuttle_url( $self->{url} . '/out_doc',
 		'application/x-www-form-urlencoded',
 		"uri=" . uri_escape($uri),
 		undef
-	) == 200) {
-		$self->_clear_info;
-		return 1;
-	}
-	return undef;
+	) == 200;
 }
 
 
@@ -1263,15 +945,11 @@ sub edit_doc {
 	my $self = shift;
 	my $doc = shift || return;
 	return unless ($self->{url} && $doc->isa('Search::Estraier::Document'));
-	if ($self->shuttle_url( $self->{url} . '/edit_doc',
+	$self->shuttle_url( $self->{url} . '/edit_doc',
 		'text/x-estraier-draft',
 		$doc->dump_draft,
 		undef
-	) == 200) {
-		$self->_clear_info;
-		return 1;
-	}
-	return undef;
+	) == 200;
 }
 
 
@@ -1380,14 +1058,12 @@ Get ID of document specified by URI
 
   my $id = $node->uri_to_id( 'file:///document/uri/42' );
 
-This method won't croak, even if using C<croak_on_error>.
-
 =cut
 
 sub uri_to_id {
 	my $self = shift;
 	my $uri = shift || return;
-	return $self->_fetch_doc( uri => $uri, path => '/uri_to_id', chomp_resbody => 1, croak_on_error => 0 );
+	return $self->_fetch_doc( uri => $uri, path => '/uri_to_id', chomp_resbody => 1 );
 }
 
 
@@ -1429,7 +1105,7 @@ sub _fetch_doc {
 	$path = '/etch_doc' if ($a->{etch});
 
 	if ($a->{id}) {
-		croak "id must be number not '$a->{id}'" unless ($a->{id} =~ m/^\d+$/);
+		croak "id must be numberm not '$a->{id}'" unless ($a->{id} =~ m/^\d+$/);
 		$arg = 'id=' . $a->{id};
 	} elsif ($a->{uri}) {
 		$arg = 'uri=' . uri_escape($a->{uri});
@@ -1447,7 +1123,6 @@ sub _fetch_doc {
 		'application/x-www-form-urlencoded',
 		$arg,
 		\$resbody,
-		$a->{croak_on_error},
 	);
 
 	return if ($rv != 200);
@@ -1478,8 +1153,8 @@ sub _fetch_doc {
 
 sub name {
 	my $self = shift;
-	$self->_set_info unless ($self->{inform}->{name});
-	return $self->{inform}->{name};
+	$self->_set_info unless ($self->{name});
+	return $self->{name};
 }
 
 
@@ -1491,8 +1166,8 @@ sub name {
 
 sub label {
 	my $self = shift;
-	$self->_set_info unless ($self->{inform}->{label});
-	return $self->{inform}->{label};
+	$self->_set_info unless ($self->{label});
+	return $self->{label};
 }
 
 
@@ -1504,8 +1179,8 @@ sub label {
 
 sub doc_num {
 	my $self = shift;
-	$self->_set_info if ($self->{inform}->{dnum} < 0);
-	return $self->{inform}->{dnum};
+	$self->_set_info if ($self->{dnum} < 0);
+	return $self->{dnum};
 }
 
 
@@ -1517,8 +1192,8 @@ sub doc_num {
 
 sub word_num {
 	my $self = shift;
-	$self->_set_info if ($self->{inform}->{wnum} < 0);
-	return $self->{inform}->{wnum};
+	$self->_set_info if ($self->{wnum} < 0);
+	return $self->{wnum};
 }
 
 
@@ -1530,8 +1205,8 @@ sub word_num {
 
 sub size {
 	my $self = shift;
-	$self->_set_info if ($self->{inform}->{size} < 0);
-	return $self->{inform}->{size};
+	$self->_set_info if ($self->{size} < 0);
+	return $self->{size};
 }
 
 
@@ -1564,32 +1239,88 @@ sub search {
 	);
 	return if ($rv != 200);
 
-	my @records 	= split /--------\[.*?\]--------(?::END)?\r?\n/, $resbody;
-	my $hintsText	= splice @records, 0, 2; # starts with empty record
-	my $hints		= { $hintsText =~ m/^(.*?)\t(.*?)$/gsm };
+	my (@docs, $hints);
 
-	# process records
-	my $docs = [];
-	foreach my $record (@records)
-	{
-		# split into keys and snippets
-		my ($keys, $snippet) = $record =~ m/^(.*?)\n\n(.*?)$/s;
+	my @lines = split(/\n/, $resbody);
+	return unless (@lines);
 
-		# create document hash
-		my $doc				= { $keys =~ m/^(.*?)=(.*?)$/gsm };
-		$doc->{'@keywords'}	= $doc->{keywords};
-		($doc->{keywords})	= $keys =~ m/^%VECTOR\t(.*?)$/gm;
-		$doc->{snippet}		= $snippet;
+	my $border = $lines[0];
+	my $isend = 0;
+	my $lnum = 1;
 
-		push @$docs, new Search::Estraier::ResultDocument(
-			attrs 		=> $doc,
-			uri 		=> $doc->{'@uri'},
-			snippet 	=> $snippet,
-			keywords 	=> $doc->{'keywords'},
-		);
+	while ( $lnum <= $#lines ) {
+		my $line = $lines[$lnum];
+		$lnum++;
+
+		#warn "## $line\n";
+		if ($line && $line =~ m/^\Q$border\E(:END)*$/) {
+			$isend = $1;
+			last;
+		}
+
+		if ($line =~ /\t/) {
+			my ($k,$v) = split(/\t/, $line, 2);
+			$hints->{$k} = $v;
+		}
 	}
 
-	return new Search::Estraier::NodeResult( docs => $docs, hints => $hints );
+	my $snum = $lnum;
+
+	while( ! $isend && $lnum <= $#lines ) {
+		my $line = $lines[$lnum];
+		#warn "# $lnum: $line\n";
+		$lnum++;
+
+		if ($line && $line =~ m/^\Q$border\E/) {
+			if ($lnum > $snum) {
+				my $rdattrs;
+				my $rdvector;
+				my $rdsnippet;
+				
+				my $rlnum = $snum;
+				while ($rlnum < $lnum - 1 ) {
+					#my $rdline = $self->_s($lines[$rlnum]);
+					my $rdline = $lines[$rlnum];
+					$rlnum++;
+					last unless ($rdline);
+					if ($rdline =~ /^%/) {
+						$rdvector = $1 if ($rdline =~ /^%VECTOR\t(.+)$/);
+					} elsif($rdline =~ /=/) {
+						$rdattrs->{$1} = $2 if ($rdline =~ /^(.+)=(.+)$/);
+					} else {
+						confess "invalid format of response";
+					}
+				}
+				while($rlnum < $lnum - 1) {
+					my $rdline = $lines[$rlnum];
+					$rlnum++;
+					$rdsnippet .= "$rdline\n";
+				}
+				#warn Dumper($rdvector, $rdattrs, $rdsnippet);
+				if (my $rduri = $rdattrs->{'@uri'}) {
+					push @docs, new Search::Estraier::ResultDocument(
+						uri => $rduri,
+						attrs => $rdattrs,
+						snippet => $rdsnippet,
+						keywords => $rdvector,
+					);
+				}
+			}
+			$snum = $lnum;
+			#warn "### $line\n";
+			$isend = 1 if ($line =~ /:END$/);
+		}
+
+	}
+
+	if (! $isend) {
+		warn "received result doesn't have :END\n$resbody";
+		return;
+	}
+
+	#warn Dumper(\@docs, $hints);
+
+	return new Search::Estraier::NodeResult( docs => \@docs, hints => $hints );
 }
 
 
@@ -1638,18 +1369,6 @@ sub cond_to_query {
 	push @args, 'wwidth=' . $self->{wwidth};
 	push @args, 'hwidth=' . $self->{hwidth};
 	push @args, 'awidth=' . $self->{awidth};
-	push @args, 'skip=' . $cond->{skip} if ($cond->{skip});
-
-	if (my $distinct = $cond->distinct) {
-		push @args, 'distinct=' . uri_escape($distinct);
-	}
-
-	if ($cond->{mask}) {
-		my $mask = 0;
-		map { $mask += ( 2 ** $_ ) } @{ $cond->{mask} };
-
-		push @args, 'mask=' . $mask if ($mask);
-	}
 
 	return join('&', @args);
 }
@@ -1672,9 +1391,7 @@ use LWP::UserAgent;
 sub shuttle_url {
 	my $self = shift;
 
-	my ($url, $content_type, $reqbody, $resbody, $croak_on_error) = @_;
-
-	$croak_on_error = $self->{croak_on_error} unless defined($croak_on_error);
+	my ($url, $content_type, $reqbody, $resbody) = @_;
 
 	$self->{status} = -1;
 
@@ -1701,7 +1418,7 @@ sub shuttle_url {
 
 	$req->headers->header( 'Host' => $url->host . ":" . $url->port );
 	$req->headers->header( 'Connection', 'close' );
-	$req->headers->header( 'Authorization', 'Basic ' . $self->{auth} ) if ($self->{auth});
+	$req->headers->header( 'Authorization', 'Basic ' . $self->{auth} );
 	$req->content_type( $content_type );
 
 	warn $req->headers->as_string,"\n" if ($self->{debug});
@@ -1715,15 +1432,9 @@ sub shuttle_url {
 
 	warn "## response status: ",$res->status_line,"\n" if ($self->{debug});
 
-	($self->{status}, $self->{status_message}) = split(/\s+/, $res->status_line, 2);
+	return -1 if (! $res->is_success);
 
-	if (! $res->is_success) {
-		if ($croak_on_error) {
-			croak("can't get $url: ",$res->status_line);
-		} else {
-			return -1;
-		}
-	}
+	($self->{status}, $self->{status_message}) = split(/\s+/, $res->status_line, 2);
 
 	$$resbody .= $res->content;
 
@@ -1796,7 +1507,7 @@ sub set_user {
 	croak "mode must be number, not '$mode'" unless ($mode =~ m/^\d+$/);
 
 	$self->shuttle_url( $self->{url} . '/_set_user',
-		'application/x-www-form-urlencoded',
+		'text/plain',
 		'name=' . uri_escape($name) . '&mode=' . $mode,
 		undef
 	) == 200;
@@ -1823,218 +1534,13 @@ sub set_link {
 	my $reqbody = 'url=' . uri_escape($url) . '&label=' . uri_escape($label);
 	$reqbody .= '&credit=' . $credit if ($credit > 0);
 
-	if ($self->shuttle_url( $self->{url} . '/_set_link',
+	$self->shuttle_url( $self->{url} . '/_set_link',
 		'application/x-www-form-urlencoded',
 		$reqbody,
 		undef
-	) == 200) {
-		# refresh node info after adding link
-		$self->_clear_info;
-		return 1;
-	}
-	return undef;
+	) == 200;
 }
 
-=head2 admins
-
- my @admins = @{ $node->admins };
-
-Return array of users with admin rights on node
-
-=cut
-
-sub admins {
-	my $self = shift;
-	$self->_set_info unless ($self->{inform}->{name});
-	return $self->{inform}->{admins};
-}
-
-=head2 guests
-
- my @guests = @{ $node->guests };
-
-Return array of users with guest rights on node
-
-=cut
-
-sub guests {
-	my $self = shift;
-	$self->_set_info unless ($self->{inform}->{name});
-	return $self->{inform}->{guests};
-}
-
-=head2 links
-
- my $links = @{ $node->links };
-
-Return array of links for this node
-
-=cut
-
-sub links {
-	my $self = shift;
-	$self->_set_info unless ($self->{inform}->{name});
-	return $self->{inform}->{links};
-}
-
-=head2 cacheusage
-
-Return cache usage for a node
-
-  my $cache = $node->cacheusage;
-
-=cut
-
-sub cacheusage {
-	my $self = shift;
-
-	return unless ($self->{url});
-
-	my $resbody;
-	my $rv = $self->shuttle_url( $self->{url} . '/cacheusage',
-		'text/plain',
-		undef,
-		\$resbody,
-	);
-
-	return if ($rv != 200 || !$resbody);
-
-	return $resbody;
-}
-
-=head2 master
-
-Set actions on Hyper Estraier node master (C<estmaster> process)
-
-  $node->master(
-  	action => 'sync'
-  );
-
-All available actions are documented in
-L<http://hyperestraier.sourceforge.net/nguide-en.html#protocol>
-
-=cut
-
-my $estmaster_rest = {
-	shutdown => {
-		status => 202,
-	},
-	sync => {
-		status => 202,
-	},
-	backup => {
-		status => 202,
-	},
-	userlist => {
-		status => 200,
-		returns => [ qw/name passwd flags fname misc/ ],
-	},
-	useradd => {
-		required => [ qw/name passwd flags/ ],
-		optional => [ qw/fname misc/ ],
-		status => 200,
-	},
-	userdel => {
-		required => [ qw/name/ ],
-		status => 200,
-	},
-	nodelist => {
-		status => 200,
-		returns => [ qw/name label doc_num word_num size/ ],
-	},
-	nodeadd => {
-		required => [ qw/name/ ],
-		optional => [ qw/label/ ],
-		status => 200,
-	},
-	nodedel => {
-		required => [ qw/name/ ],
-		status => 200,
-	},
-	nodeclr => {
-		required => [ qw/name/ ],
-		status => 200,
-	},
-	nodertt => {
-		status => 200,	
-	},
-};
-
-sub master {
-	my $self = shift;
-
-	my $args = {@_};
-
-	# have action?
-	my $action = $args->{action} || croak "need action, available: ",
-		join(", ",keys %{ $estmaster_rest });
-
-	# check if action is valid
-	my $rest = $estmaster_rest->{$action};
-	croak "action '$action' is not supported, available actions: ",
-		join(", ",keys %{ $estmaster_rest }) unless ($rest);
-
-	croak "BUG: action '$action' needs return status" unless ($rest->{status});
-
-	my @args;
-
-	if ($rest->{required} || $rest->{optional}) {
-
-		map {
-			croak "need parametar '$_' for action '$action'" unless ($args->{$_});
-			push @args, $_ . '=' . uri_escape( $args->{$_} );
-		} ( @{ $rest->{required} } );
-
-		map {
-			push @args, $_ . '=' . uri_escape( $args->{$_} ) if ($args->{$_});
-		} ( @{ $rest->{optional} } );
-
-	}
-
-	my $uri = new URI( $self->{url} );
-
-	my $resbody;
-
-	my $status = $self->shuttle_url(
-		'http://' . $uri->host_port . '/master?action=' . $action ,
-		'application/x-www-form-urlencoded',
-		join('&', @args),
-		\$resbody,
-		1,
-	) or confess "shuttle_url failed";
-
-	if ($status == $rest->{status}) {
-
-		# refresh node info after sync
-		$self->_clear_info if ($action eq 'sync' || $action =~ m/^node(?:add|del|clr)$/);
-
-		if ($rest->{returns} && wantarray) {
-
-			my @results;
-			my $fields = $#{$rest->{returns}};
-
-			foreach my $line ( split(/[\r\n]/,$resbody) ) {
-				my @e = split(/\t/, $line, $fields + 1);
-				my $row;
-				foreach my $i ( 0 .. $fields) {
-					$row->{ $rest->{returns}->[$i] } = $e[ $i ];
-				}
-				push @results, $row;
-			}
-
-			return @results;
-
-		} elsif ($resbody) {
-			chomp $resbody;
-			return $resbody;
-		} else {
-			return 0E0;
-		}
-	}
-
-	carp "expected status $rest->{status}, but got $status";
-	return undef;
-}
 
 =head1 PRIVATE METHODS
 
@@ -2063,50 +1569,12 @@ sub _set_info {
 
 	return if ($rv != 200 || !$resbody);
 
-	my @lines = split(/[\r\n]/,$resbody);
+	# it seems that response can have multiple line endings
+	$resbody =~ s/[\r\n]+$//;
 
-	$self->_clear_info;
+	( $self->{name}, $self->{label}, $self->{dnum}, $self->{wnum}, $self->{size} ) =
+		split(/\t/, $resbody, 5);
 
-	( $self->{inform}->{name}, $self->{inform}->{label}, $self->{inform}->{dnum},
-		$self->{inform}->{wnum}, $self->{inform}->{size} ) = split(/\t/, shift @lines, 5);
-
-	return $resbody unless (@lines);
-
-	shift @lines;
-
-	while(my $admin = shift @lines) {
-		push @{$self->{inform}->{admins}}, $admin;
-	}
-
-	while(my $guest = shift @lines) {
-		push @{$self->{inform}->{guests}}, $guest;
-	}
-
-	while(my $link = shift @lines) {
-		push @{$self->{inform}->{links}}, $link;
-	}
-
-	return $resbody;
-
-}
-
-=head2 _clear_info
-
-Clear information for node
-
-  $node->_clear_info;
-
-On next call to C<name>, C<label>, C<doc_num>, C<word_num> or C<size> node
-info will be fetch again from Hyper Estraier.
-
-=cut
-sub _clear_info {
-	my $self = shift;
-	$self->{inform} = {
-		dnum => -1,
-		wnum => -1,
-		size => -1.0,
-	};
 }
 
 ###
@@ -2121,15 +1589,10 @@ L<http://hyperestraier.sourceforge.net/>
 
 Hyper Estraier Ruby interface on which this module is based.
 
-Hyper Estraier now also has pure-perl binding included in distribution. It's
-a faster way to access databases directly if you are not running
-C<estmaster> P2P server.
-
 =head1 AUTHOR
 
 Dobrica Pavlinusic, E<lt>dpavlin@rot13.orgE<gt>
 
-Robert Klep E<lt>robert@klep.nameE<gt> contributed refactored search code
 
 =head1 COPYRIGHT AND LICENSE
 
